@@ -14,8 +14,8 @@ from copy import deepcopy
 class Dumper:
     """Interface for dumpers."""
 
-    def dump(self, record):
-        """Dump a record that can be used a source document for Elasticsearch.
+    def dump(self, record, data):
+        """Dump a record that can be used a source document for the search engine.
 
         The job of this method is to create a Python dictionary from the record
         provided in the argument.
@@ -25,11 +25,13 @@ class Dumper:
         that changes to the dump affects the record.
 
         :param record: The record to dump.
+        :param data: The initial dump data passed in by ``record.dumps()``.
         """
-        return deepcopy(dict(record))
+        data.update(deepcopy(dict(record)))
+        return data
 
     def load(self, data, record_cls):
-        """Load a record from the source document of an Elasticsearch hit.
+        """Load a record from the source document of a search engine hit.
 
         The job of this method, is to create a record of type ``record_cls``
         based on the input ``data``.
