@@ -13,6 +13,7 @@ import pytest
 from flask import Flask
 from invenio_celery import InvenioCelery
 from invenio_db import InvenioDB
+from invenio_i18n import InvenioI18N
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.schema import DropConstraint, DropSequence, DropTable
 
@@ -51,6 +52,7 @@ def create_app(instance_path):
         InvenioCelery(app_)
         InvenioDB(app_)
         InvenioRecords(app_)
+        InvenioI18N(app_)
         return app_
 
     return _create_app
