@@ -14,10 +14,10 @@ from invenio_db.utils import drop_alembic_version_table
 
 def test_alembic(testapp, db):
     """Test alembic recipes."""
-    ext = testapp.extensions['invenio-db']
+    ext = testapp.extensions["invenio-db"]
 
-    if db.engine.name == 'sqlite':
-        raise pytest.skip('Upgrades are not supported on SQLite.')
+    if db.engine.name == "sqlite":
+        raise pytest.skip("Upgrades are not supported on SQLite.")
 
     assert not ext.alembic.compare_metadata()
     db.drop_all()
@@ -26,7 +26,7 @@ def test_alembic(testapp, db):
 
     assert not ext.alembic.compare_metadata()
     ext.alembic.stamp()
-    ext.alembic.downgrade(target='96e796392533')
+    ext.alembic.downgrade(target="96e796392533")
     ext.alembic.upgrade()
 
     assert not ext.alembic.compare_metadata()
